import React from 'react';
import { NodeType } from '../../nodelinkvis/types';
import { GeoJsonType } from '../mapvis.types';
import { SearchResultType } from '../mapvis.types';

export type NodelinkPopupProps = {
  type: 'node' | 'area' | 'location';
  data: NodeType | GeoJsonType | SearchResultType;
  onClose: () => void;
};

const isGeoJsonType = (data: NodeType | GeoJsonType | SearchResultType): data is GeoJsonType => {
  return (data as GeoJsonType).properties !== undefined;
};

export const MapTooltip = (props: NodelinkPopupProps) => {
  const { type, data } = props;

  const renderNodeDetails = (node: NodeType) => (
    <div>
      {node.attributes &&
        Object.entries(node.attributes).map(([k, v]) => (
          <div key={k} className="flex flex-row gap-3">
            <span>{k}: </span>
            <span className="ml-auto max-w-[10rem] text-right truncate">
              <span title={JSON.stringify(v)}>{JSON.stringify(v)}</span>
            </span>
          </div>
        ))}
    </div>
  );

  const renderAreaDetails = (geoJson: GeoJsonType) => (
    <div>
      <div className="flex flex-row gap-3">
        <span>Area id: </span>
        <span className="ml-auto max-w-[10rem] text-right truncate">
          <span>{geoJson.properties?.regioFacetId ?? 'N/A'}</span>
        </span>
      </div>
      <div className="flex flex-row gap-3">
        <span>Nodes in area: </span>
        <span className="ml-auto max-w-[10rem] text-right truncate">
          <span>{geoJson.properties?.nodes?.length ?? 0}</span>
        </span>
      </div>
      <div className="flex flex-row gap-3">
        <span>Townships in area: </span>
        <span className="ml-auto max-w-[10rem] text-right truncate">
          <span>{geoJson.properties?.townships?.length ?? 0}</span>
        </span>
      </div>
    </div>
  );

  const renderLocationDetails = (location: SearchResultType) => (
    <div>
      <div className="flex flex-row gap-3">
        <span>Name: </span>
        <span className="ml-auto max-w-[10rem] text-right truncate">
          <span>{location?.display_name}</span>
        </span>
      </div>
      <div className="flex flex-row gap-3">
        <span>Type: </span>
        <span className="ml-auto max-w-[10rem] text-right truncate">
          <span>{location?.addresstype}</span>
        </span>
      </div>
      <div className="flex flex-row gap-3">
        <span>Coordinate: </span>
        <span className="ml-auto max-w-[10rem] text-right truncate">
          <span>
            [{location?.lon}, {location?.lat}]
          </span>
        </span>
      </div>
    </div>
  );

  return (
    <div className="text-[0.9rem] min-w-[10rem]">
      <div className="card-body p-0">
        <span className="px-2.5 pt-2">
          <span>{type === 'node' ? 'Node' : 'Area'}</span>
          <span className="float-right">
            {type === 'node'
              ? (data as NodeType)?._id
              : isGeoJsonType(data)
                ? data.properties?.name
                : type === 'location'
                  ? (data as SearchResultType)?.name
                  : 'N/A'}
          </span>
        </span>
        <div className="h-[1px] w-full bg-secondary-200"></div>
        <div className="px-2.5 text-[0.8rem]">
          {type === 'node'
            ? data && 'attributes' in data
              ? renderNodeDetails(data as NodeType)
              : null
            : data && isGeoJsonType(data)
              ? renderAreaDetails(data as GeoJsonType)
              : renderLocationDetails(data as SearchResultType)}
        </div>
        <div className="h-[1px] w-full"></div>
      </div>
    </div>
  );
};
